@@ -31,48 +31,79 @@ use feature           qw( say state );
 use parent            qw( Exporter );
 use subs              qw( p uniq );
 
-our $VERSION = '0.06';
-our @EXPORT  = qw( run h p );
+our $VERSION = '0.07';
+our @EXPORT  = qw( run p );
 
 =head1 NAME
 
 Runtime::Debugger - Easy to use REPL with existing lexicals support.
 
+(empahsis on "existing" since I have not yet found this support in others modules).
+
 =head1 SYNOPSIS
 
-tl;dr - Easy to use REPL with existing lexicals support.
+Start the debugger:
 
-(empahsis on "existing" since I have not yet found this support
-in others modules).
+    perl -MRuntime::Debugger -E 'eval run'
 
-Try with this command line:
+Same, but with some variables to play with:
 
- perl -MRuntime::Debugger -E 'my $str1 = "Func"; our $str2 = "Func2"; my @arr1 = "arr-1"; our @arr2 = "arr-2"; my %hash1 = qw(hash 1); our %hash2 = qw(hash 2); my $coderef = sub { "code-ref: @_" }; {package My; sub Func{"My-Func"} sub Func2{"My-Func2"}} my $obj = bless {}, "My"; eval run; say $@'
+    perl -MRuntime::Debugger -E 'my $str1 = "Func"; our $str2 = "Func2"; my @arr1 = "arr-1"; our @arr2 = "arr-2"; my %hash1 = qw(hash 1); our %hash2 = qw(hash 2); my $coderef = sub { "code-ref: @_" }; {package My; sub Func{"My-Func"} sub Func2{"My-Func2"}} my $obj = bless {}, "My"; eval run; say $@'
 
 =head1 DESCRIPTION
 
-One can usually just do this:
+"What? Another debugger? What about ... ?"
 
- # Insert this where you want to pause:
- $DB::single = 1;
+=head2 perl5db.pl
 
- # Then run the perl debugger to navigate there quickly:
- PERLDBOPT='Nonstop' perl -d my_script
+The standard perl debugger (C<perl5db.pl>) is a powerful tool.
 
-If that works for then great and dont' bother using this module!
+Using C<per5db.pl>, one would normally be able to do this:
 
-Unfortunately for me, it was not working due to the scenario
-in which a script evals another perl test file and I would have
-liked to pause inside the test and see whats going on without
-having to keep rerunning the whole test over and over.
+    # Insert a breakpoint in your code:
+    $DB::single = 1;
 
-This module basically drops in a read,evaludate,print loop (REPL)
-whereever you need like so:
+    # Then run the perl debugger to navigate there quickly:
+    PERLDBOPT='Nonstop' perl -d my_script
+
+If that works for you, then dont' bother with this module!
+(joke. still try it.)
+
+=head2 Dilemma
+
+I have this scenario:
+
+ - A perl script gets executed.
+ - The script calls a support module.
+ - The module reads a test file.
+ - The module string evals the string contents of the test file.
+ - The test takes possibly minutes to run (Selenium).
+ - The test is failing.
+ - Not sure what is failing.
+
+Normal workflow would be:
+
+ - Step 1: Apply a fix.
+ - Step 2: Run the test.
+ - Step 3: Wait ... wait ... wait.
+ - Step 4: Goto Step 1 if test fails.
+
+=head2 Solution
+
+This module basically inserts a read, evaluate, print loop (REPL)
+wherever you need it.
 
     use Runtime::Debugger;
-    eval run;                # Not sure how to avoid using eval here while
-                             # also being able to keep the lexical scope.
-                             # Any ideas ? :)
+    eval run;
+
+=head2 Tab Completion
+
+The module support rich tab completion support:
+
+ - Press TAB with no input to view commands and available variables in the current scope.
+ -
+
+TODO
 
 Press tab to autocomplete any lexical variables in scope (where "eval run" is found).
 
@@ -80,6 +111,16 @@ Saves history locally.
 
 Can use 'p' to pretty print a variable or structure.
 
+
+
+=head2 Ideas
+
+Not sure how to avoid using eval here while keeping access to the top level lexical scope.
+
+(Maybe through abuse of PadWalker and modifying input dynamically.)
+
+Any ideas ? :)
+
 =head2 New Variables
 
 Currently its not possible to create any new lexicals variables
@@ -88,9 +129,9 @@ while I have not yet found a way to run "eval" with a higher scope of lexicals.
 
 You can make global variables though if:
 
-- By default ($var=123)
-- Using our (our $var=123)
-- Given the full path ($My::var = 123)
+ - By default ($var=123)
+ - Using our (our $var=123)
+ - Given the full path ($My::var = 123)
 
 =head1 SUBROUTINES/METHODS
 
@@ -110,6 +151,8 @@ Sets C<$@> to the exit reason like 'INT' (Control-C) or 'q' (Normal exit/quit).
 
 sub run {
     <<'CODE';
+    use strict;
+    use warnings;
     my $repl = Runtime::Debugger->_init;
     while ( 1 ) {
         eval $repl->_step;
@@ -195,7 +238,7 @@ sub _complete_empty {
     $self->_dump_args( @_ ) if $self->debug;
 
     $self->_match(
-        words   => $self->{commands_and_variables},
+        words   => $self->{commands_and_vars_all},
         partial => $text,
     );
 }
@@ -306,10 +349,11 @@ sub _complete_vars {
 Returns the possible matches:
 
 Input:
- words   => ["cat", "cake", "bat", "bake"],
- partial => "c",            # Default: ""  - What you typed so far.
- prepend => "Class>",       # Default: ""  - prepend to each possiblity.
- nospace => 0,              # Default: "0" - will not append a space after a completion.
+
+ words   => ARRAYREF, # What to look for.
+ partial => STRING,   # Default: ""  - What you typed so far.
+ prepend => "STRING", # Default: ""  - prepend to each possiblity.
+ nospace => 0,        # Default: "0" - will not append a space after a completion.
 
 =cut
 
@@ -341,7 +385,7 @@ sub _define_commands {
         "help",    # Changed in _step to $repl->help().
         "hist",    # Changed in _step to $repl->hist().
         "p",       # Exporting it.
-        "q",       # Exporting it.
+        "q",       # Used in _step to stop the repl.
     );
 }
 
@@ -380,8 +424,8 @@ sub _setup_vars {
 
     my @commands = $self->_define_commands;
 
-    $self->{commands}               = \@commands;
-    $self->{commands_and_variables} = [ sort( @commands, @vars_all ) ];
+    $self->{commands}              = \@commands;
+    $self->{commands_and_vars_all} = [ sort( @commands, @vars_all ) ];
 
     # Separate variables by types.
     for ( @vars_all ) {
@@ -401,6 +445,12 @@ sub _setup_vars {
                 elsif ( ref( $$ref ) eq "CODE" ) {
                     push @{ $self->{vars_code} }, $_;
                 }
+                elsif ( ref( $$ref ) eq "HASH" ) {
+                    push @{ $self->{vars_hashref} }, $_;
+                }
+                elsif ( ref( $$ref ) eq "ARRAY" ) {
+                    push @{ $self->{vars_arrayref} }, $_;
+                }
                 else {
                     push @{ $self->{vars_ref_else} }, $_;
                 }
@@ -426,7 +476,10 @@ sub _setup_vars {
 sub _step {
     my ( $self ) = @_;
 
-    $self->_setup_vars if not $self->{vars_all};
+    if ( not $self->{vars_all} ) {
+        $self->_setup_vars;
+        $self->help;    # Show help when first loading the debugger.
+    }
 
     my $input = $self->term->readline( "perl>" ) // '';
     say "input_after_readline=[$input]" if $self->debug;
@@ -609,13 +662,7 @@ sub p {
     print $d->Dump;
 }
 
-# Unique
-
-=head2 uniq
-
-Return a list of uniq values.
-
-=cut
+# List Utils.
 
 sub uniq (@) {
     my %h;
@@ -687,7 +734,7 @@ Enable this environmental variable to show debugging information:
 Great extendable module!
 
 Unfortunately, I did not find a way to get the lexical variables
-in a scope. (maybe missed a plugin?!)
+in a scope. (maybe I missed a plugin?!)
 
 =head2 L<https://metacpan.org/pod/Reply>
 
@@ -699,7 +746,7 @@ Tim Potapov, C<< <tim.potapov[AT]gmail.com> >> E<0x1f42a>E<0x1f977>
 
 =head1 BUGS
 
-- L<no new lexicals|/=head2 New Variables>
+- L<no new lexicals|/New Variables>
 
 Please report any (other) bugs or feature requests to L<https://github.com/poti1/runtime-debugger/issues>.
 
@@ -713,7 +760,7 @@ You can find documentation for this module with the perldoc command.
 
 You can also look for information at:
 
-L<https://metacpan.org/Runtime::Debugger>
+L<https://metacpan.org/pod/Runtime::Debugger>
 L<https://github.com/poti1/runtime-debugger>
 
 
